import {
  getContext,
  getContextItem,
  setContextItem,
  setDefaultContext,
} from "./lib/context.ts";
import { InputManager } from "./lib/input.ts";

import { Doodler, Vector, ZoomableDoodler } from "@bearmetal/doodler";
import { ResourceManager } from "./lib/resources.ts";
import { addButton } from "./ui/button.ts";
import { TrackSystem } from "./track/system.ts";
import { StraightTrack } from "./track/shapes.ts";
import { State, StateMachine } from "./state/machine.ts";
import { bootstrapGameStateMachine } from "./state/states/index.ts";
import { GameLoop } from "./GameLoop.ts";

const inputManager = new InputManager();
const resources = new ResourceManager();
const doodler = new ZoomableDoodler({
  fillScreen: true,
  bg: "#302040",
  noSmooth: true,
}, () => {});
setTimeout(() => {
  (doodler as any as { ctx: CanvasRenderingContext2D }).ctx
    .imageSmoothingEnabled = false;
}, 0);
// doodler.minScale = 0.1;
(doodler as any).scale = 3.14;

const colors = [
  "red",
  "orange",
  "yellow",
  "green",
  "blue",
  "indigo",
  "purple",
  "violet",
];

const _fullDebug: Debug = {
  track: false,
  train: false,
  path: false,
  car: false,
  bogies: false,
  angles: false,
  aabb: false,
};

const storedDebug = JSON.parse(localStorage.getItem("debug") || "0");
const _debug: Debug = Object.assign({}, _fullDebug, storedDebug);

const debug = new Proxy(_debug, {
  get: (_, prop: string) => {
    // if (prop !in _debug) {
    //   (_debug as any)[prop] = false;
    // }
    return prop in _debug ? (_debug as any)[prop] : false;
  },
  set: (_, prop: string, value: boolean) => {
    (_debug as any)[prop] = value;
    localStorage.setItem("debug", JSON.stringify(_debug));
    return true;
  },
});

setDefaultContext({
  inputManager,
  doodler,
  resources,
  debug,
  colors,
});

const state = bootstrapGameStateMachine();
setContextItem("state", state);

doodler.init();

// doodler.createLayer((_, __, dTime) => {
//   state.update(dTime);
// });

document.addEventListener("keydown", (e) => {
  if ((e.ctrlKey || e.metaKey) && e.key === "s") {
    e.preventDefault();
    const track = getContextItem<TrackSystem>("track");
    localStorage.setItem("track", track.serialize());
    console.log("Saved track to local storage");
  }
});

setInterval(() => {
  const doodler = getContextItem<Doodler>("doodler");
  const frameRate = doodler.fps;
  if (frameRate < 0.5) return;
  let fpsEl = document.getElementById("fps");
  if (!fpsEl) {
    fpsEl = document.createElement("div");
    fpsEl.id = "fps";
    document.body.appendChild(fpsEl);
  }
  // const fPerc = frameRate / 60;
  // if (fPerc < 0.6) {
  //   state.optimizePerformance(fPerc);
  // }
  fpsEl.textContent = frameRate.toFixed(1) + " fps";
}, 1000);

const gameLoop = new GameLoop();
gameLoop.start(state);

if (import.meta.env.DEV) {
  console.log("Running in development mode");
}

globalThis.TWO_PI = Math.PI * 2;
declare global {
  var TWO_PI: number;
}
